# Copyright (C) 2019  The Software Heritage developers
# See the AUTHORS file at the top-level directory of this distribution
# License: GNU General Public License version 3, or any later version
# See top-level LICENSE file for more information

# the default prefix for kafka's topics
DEFAULT_PREFIX = 'swh.journal.objects'
